﻿#region Using directives

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Text;
using System.Windows.Forms;

#endregion

namespace RegionsSample
{
    partial class ClippingRegionForm : Form
    {
        public ClippingRegionForm()
        {
            InitializeComponent();

            //this.SetStyle(ControlStyles.ResizeRedraw, true);
            //this.SetStyle(ControlStyles.OptimizedDoubleBuffer, true);
            // this.SetStyle(ControlStyles.AllPaintingInWmPaint, true);
        }

        Region region;
        private void SetRegion()
        {
            if (region != null) region.MakeEmpty();
            switch (shape)
            {
                case Shape.Ellipse:
                    region = CreateEllipse();
                    break;

                case Shape.Doughnut:
                    region = CreateDoughnut();
                    break;
            }

            panelDraw.Invalidate();
        }

        private Region CreateEllipse()
        {
            Region regionOuter;

            using (GraphicsPath outerPath = new GraphicsPath())
            {
                outerPath.AddEllipse(panelDraw.ClientRectangle);
                regionOuter = new Region(outerPath);
            }
            return regionOuter;
        }

        private Region CreateDoughnut()
        {
            Region regionOuter;

            Rectangle midRect = panelDraw.ClientRectangle;
            Rectangle innerRect = panelDraw.ClientRectangle;
            midRect.Inflate(-20, -20);
            innerRect.Inflate(-40, -40);

            using (GraphicsPath outerPath = new GraphicsPath())
            using (GraphicsPath innerPath = new GraphicsPath())
            using (GraphicsPath midPath = new GraphicsPath())
            {
                outerPath.AddEllipse(panelDraw.ClientRectangle);
                innerPath.AddEllipse(innerRect);
                midPath.AddEllipse(midRect);
                regionOuter = new Region(outerPath);
                regionOuter.Exclude(midPath);
                regionOuter.Union(innerPath);

            }

            return regionOuter;
        }

        private void ClippingRegionForm_Paint(object sender, PaintEventArgs e)
        {
            Graphics g = e.Graphics;
            StringFormat format = new StringFormat();
            format.Alignment = StringAlignment.Center;
            format.LineAlignment = StringAlignment.Center;

            // Frame clipping region
            g.DrawEllipse(Pens.Red, panelDraw.ClientRectangle);

            // Don't draw outside the ellipse region
            g.Clip = region;

            // Draw a rectangle
            Rectangle rect = panelDraw.ClientRectangle;
            rect.Offset(10, 10);
            rect.Width -= 20;
            rect.Height -= 20;
            g.FillRectangle(Brushes.Black, rect);
            String text = String.Format("Rectangle clipped to {0}", Enum.GetName(typeof(Shape), shape));
            g.DrawString(text, this.Font, Brushes.White, rect, format);


        }

        enum Shape { Ellipse, Doughnut }
        Shape shape = Shape.Ellipse;
        private void ellipseToolStripMenuItem_Click(object sender, EventArgs e)
        {
            shape = Shape.Ellipse;
            SetRegion();
        }

        private void doughnutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            shape = Shape.Doughnut;
            SetRegion();
        }

        private void ClippingRegionForm_Load(object sender, EventArgs e)
        {
            SetRegion();
        }

        private void ClippingRegionForm_Resize(object sender, EventArgs e)
        {
            SetRegion();
        }
    }
}